@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2020 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -23,6 +23,13 @@
  * along with this program; see the file COPYING.  If not, write to
  * the Free Software Foundation, 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
+ *
+ *
+ * TODO This still needs much work to make it usable, and maintanable!
+ * - Create OdbBackendCustomType that inherits from OdbBackendType.
+ *   OdbBackendType should not be subclassed, instead subclass
+ *   OdbBackendCustomType to develop custom backends in Python.
+ *   Implement this new type in src/odb_backend_custom.c
  */
 
 #define PY_SSIZE_T_CLEAN
@@ -37,45 +44,29 @@
 #include <git2/sys/odb_backend.h>
 
 /*
- * pygit2_odb_backend is a container for the state associated with a custom
- * implementation of git_odb_backend. It holds a list of callable references
- * which represent the Python class's implementations of each git_odb_backend
- * function. The git_odb_backend field's function pointers are assigned to the
- * pygit2_odb_backend_* functions, which handle translating between the libgit2
- * ABI and the Python ABI.
+ * pgit_odb_backend_t is a container for the state associated with a custom
+ * implementation of git_odb_backend. The git_odb_backend field's function
+ * pointers are assigned to the pgit_odb_backend_* functions, which handle
+ * translating between the libgit2 ABI and the Python ABI.
+ * It holds a pointer to the <OdbBackend> subclass, which must implement
+ * the callbacks in Python.
  */
-struct pygit2_odb_backend
-{
+typedef struct {
     git_odb_backend backend;
-    PyObject *OdbBackend;
-    PyObject *read,
-             *read_prefix,
-             *read_header,
-             *write,
-             *writestream,
-             *readstream,
-             *exists,
-             *exists_prefix,
-             *refresh,
-             *writepack,
-             *freshen;
-};
+    PyObject *py_backend;
+} pgit_odb_backend;
 
 static int
-pygit2_odb_backend_read(void **ptr, size_t *sz,
-        git_object_t *type, git_odb_backend *_be, const git_oid *oid)
+pgit_odb_backend_read(void **ptr, size_t *sz, git_object_t *type,
+                      git_odb_backend *_be, const git_oid *oid)
 {
-    PyObject *args, *py_oid, *result;
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
 
-    py_oid = git_oid_to_python(oid);
+    PyObject *py_oid = git_oid_to_python(oid);
     if (py_oid == NULL)
         return GIT_EUSER;
 
-    args = Py_BuildValue("(N)", py_oid);
-    result = PyObject_CallObject(be->read, args);
-    Py_DECREF(args);
-
+    PyObject *result = PyObject_CallMethod(be->py_backend, "read_cb", "N", py_oid);
     if (result == NULL)
         return git_error_for_exc();
 
@@ -97,32 +88,27 @@ pygit2_odb_backend_read(void **ptr, size_t *sz,
 }
 
 static int
-pygit2_odb_backend_read_prefix(git_oid *oid_out, void **ptr, size_t *sz,
-        git_object_t *type, git_odb_backend *_be,
-        const git_oid *short_oid, size_t len)
+pgit_odb_backend_read_prefix(git_oid *oid_out, void **ptr, size_t *sz, git_object_t *type,
+                             git_odb_backend *_be, const git_oid *short_id, size_t len)
 {
-    PyObject *args, *py_oid, *py_oid_out, *result;
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
-
-    py_oid = git_oid_to_python(short_oid);
-    if (py_oid == NULL)
-        return GIT_EUSER;
-
-    args = Py_BuildValue("(N)", py_oid);
-    result = PyObject_CallObject(be->read_prefix, args);
-    Py_DECREF(args);
+    // short_id to hex
+    char short_id_hex[GIT_OID_HEXSZ];
+    git_oid_nfmt(short_id_hex, len, short_id);
 
+    // Call callback
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
+    PyObject *result = PyObject_CallMethod(be->py_backend, "read_prefix_cb", "s#", short_id_hex, len);
     if (result == NULL)
         return git_error_for_exc();
 
+    // Parse output from calback
+    PyObject *py_oid_out;
     const char *bytes;
-    if (!PyArg_ParseTuple(result, "Ony#",
-                &py_oid_out, type, &bytes, sz) || !bytes) {
+    if (!PyArg_ParseTuple(result, "ny#O", type, &bytes, sz, &py_oid_out) || !bytes) {
         Py_DECREF(result);
         return GIT_EUSER;
     }
 
-
     *ptr = git_odb_backend_data_alloc(_be, *sz);
     if (!*ptr) {
         Py_DECREF(result);
@@ -136,20 +122,16 @@ pygit2_odb_backend_read_prefix(git_oid *oid_out, void **ptr, size_t *sz,
 }
 
 static int
-pygit2_odb_backend_read_header(size_t *len, git_object_t *type,
-        git_odb_backend *_be, const git_oid *oid)
+pgit_odb_backend_read_header(size_t *len, git_object_t *type,
+                             git_odb_backend *_be, const git_oid *oid)
 {
-    PyObject *args, *py_oid, *result;
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
 
-    py_oid = git_oid_to_python(oid);
+    PyObject *py_oid = git_oid_to_python(oid);
     if (py_oid == NULL)
         return GIT_EUSER;
 
-    args = Py_BuildValue("(N)", py_oid);
-    result = PyObject_CallObject(be->read_header, args);
-    Py_DECREF(args);
-
+    PyObject *result = PyObject_CallMethod(be->py_backend, "read_header_cb", "N", py_oid);
     if (result == NULL)
         return git_error_for_exc();
 
@@ -163,20 +145,16 @@ pygit2_odb_backend_read_header(size_t *len, git_object_t *type,
 }
 
 static int
-pygit2_odb_backend_write(git_odb_backend *_be, const git_oid *oid,
+pgit_odb_backend_write(git_odb_backend *_be, const git_oid *oid,
         const void *data, size_t sz, git_object_t typ)
 {
-    PyObject *args, *py_oid, *result;
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
 
-    py_oid = git_oid_to_python(oid);
+    PyObject *py_oid = git_oid_to_python(oid);
     if (py_oid == NULL)
         return GIT_EUSER;
 
-    args = Py_BuildValue("(Ny#n)", py_oid, data, sz, typ);
-    result = PyObject_CallObject(be->write, args);
-    Py_DECREF(args);
-
+    PyObject *result = PyObject_CallMethod(be->py_backend, "write_cb", "Ny#n", py_oid, data, sz, typ);
     if (result == NULL)
         return git_error_for_exc();
 
@@ -185,19 +163,15 @@ pygit2_odb_backend_write(git_odb_backend *_be, const git_oid *oid,
 }
 
 static int
-pygit2_odb_backend_exists(git_odb_backend *_be, const git_oid *oid)
+pgit_odb_backend_exists(git_odb_backend *_be, const git_oid *oid)
 {
-    PyObject *args, *py_oid, *result;
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
 
-    py_oid = git_oid_to_python(oid);
+    PyObject *py_oid = git_oid_to_python(oid);
     if (py_oid == NULL)
         return GIT_EUSER;
 
-    args = Py_BuildValue("(N)", py_oid);
-    result = PyObject_CallObject(be->exists, args);
-    Py_DECREF(args);
-
+    PyObject *result = PyObject_CallMethod(be->py_backend, "exists_cb", "N", py_oid);
     if (result == NULL)
         return git_error_for_exc();
 
@@ -207,55 +181,40 @@ pygit2_odb_backend_exists(git_odb_backend *_be, const git_oid *oid)
 }
 
 static int
-pygit2_odb_backend_exists_prefix(git_oid *out, git_odb_backend *_be,
-        const git_oid *partial, size_t len)
+pgit_odb_backend_exists_prefix(git_oid *out, git_odb_backend *_be,
+                               const git_oid *short_id, size_t len)
 {
-    PyObject *args, *py_oid, *py_oid_out, *result;
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
+    // short_id to hex
+    char short_id_hex[GIT_OID_HEXSZ];
+    git_oid_nfmt(short_id_hex, len, short_id);
 
-    py_oid = git_oid_to_python(partial);
+    // Call callback
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
+    PyObject *py_oid = PyObject_CallMethod(be->py_backend, "exists_prefix_cb", "s#", short_id_hex, len);
     if (py_oid == NULL)
-        return GIT_EUSER;
-
-    args = Py_BuildValue("(N)", py_oid);
-    result = PyObject_CallObject(be->exists_prefix, args);
-    Py_DECREF(args);
-
-    if (result == NULL)
         return git_error_for_exc();
 
-    if (!PyArg_ParseTuple(result, "O", &py_oid_out)) {
-        Py_DECREF(result);
-        return GIT_EUSER;
-    }
-
-    Py_DECREF(result);
-
-    if (py_oid_out == Py_None) {
-        return GIT_ENOTFOUND;
-    }
-
-    py_oid_to_git_oid(py_oid_out, out);
-    Py_DECREF(py_oid_out);
+    py_oid_to_git_oid(py_oid, out);
+    Py_DECREF(py_oid);
     return 0;
 }
 
 static int
-pygit2_odb_backend_refresh(git_odb_backend *_be)
+pgit_odb_backend_refresh(git_odb_backend *_be)
 {
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
-    PyObject_CallObject(be->exists_prefix, NULL);
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
+    PyObject_CallMethod(be->py_backend, "refresh_cb", NULL);
     return git_error_for_exc();
 }
 
 static int
-pygit2_odb_backend_foreach(git_odb_backend *_be,
+pgit_odb_backend_foreach(git_odb_backend *_be,
         git_odb_foreach_cb cb, void *payload)
 {
     PyObject *item;
     git_oid oid;
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
-    PyObject *iterator = PyObject_GetIter((PyObject *)be->OdbBackend);
+    pgit_odb_backend *be = (pgit_odb_backend *)_be;
+    PyObject *iterator = PyObject_GetIter((PyObject *)be->py_backend);
     assert(iterator);
 
     while ((item = PyIter_Next(iterator))) {
@@ -268,130 +227,59 @@ pygit2_odb_backend_foreach(git_odb_backend *_be,
 }
 
 static void
-pygit2_odb_backend_free(git_odb_backend *_be)
+pgit_odb_backend_free(git_odb_backend *backend)
 {
-    struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)_be;
-    Py_DECREF(be->OdbBackend);
+    pgit_odb_backend *custom_backend = (pgit_odb_backend *)backend;
+    Py_DECREF(custom_backend->py_backend);
 }
 
 int
 OdbBackend_init(OdbBackend *self, PyObject *args, PyObject *kwds)
 {
+    // Check input arguments
     if (args && PyTuple_Size(args) > 0) {
-        PyErr_SetString(PyExc_TypeError,
-                        "OdbBackend takes no arguments");
+        PyErr_SetString(PyExc_TypeError, "OdbBackend takes no arguments");
         return -1;
     }
 
     if (kwds && PyDict_Size(kwds) > 0) {
-        PyErr_SetString(PyExc_TypeError,
-                        "OdbBackend takes no keyword arguments");
+        PyErr_SetString(PyExc_TypeError, "OdbBackend takes no keyword arguments");
         return -1;
     }
 
-    struct pygit2_odb_backend *be = calloc(1, sizeof(struct pygit2_odb_backend));
-    be->backend.version = GIT_ODB_BACKEND_VERSION;
-    be->OdbBackend = (PyObject *)self;
+    // Create the C backend
+    pgit_odb_backend *custom_backend = calloc(1, sizeof(pgit_odb_backend));
+    custom_backend->backend.version = GIT_ODB_BACKEND_VERSION;
 
-    if (PyObject_HasAttrString((PyObject *)self, "read")) {
-        be->read = PyObject_GetAttrString((PyObject *)self, "read");
-        be->backend.read = pygit2_odb_backend_read;
-    }
+    // Fill the member methods
+    custom_backend->backend.free = pgit_odb_backend_free;
+    custom_backend->backend.read = pgit_odb_backend_read;
+    custom_backend->backend.read_prefix = pgit_odb_backend_read_prefix;
+    custom_backend->backend.read_header = pgit_odb_backend_read_header;
+    custom_backend->backend.write = pgit_odb_backend_write;
+    custom_backend->backend.exists = pgit_odb_backend_exists;
+    custom_backend->backend.exists_prefix = pgit_odb_backend_exists_prefix;
+    custom_backend->backend.refresh = pgit_odb_backend_refresh;
+//  custom_backend->backend.writepack = pgit_odb_backend_writepack;
+//  custom_backend->backend.freshen = pgit_odb_backend_freshen;
+//  custom_backend->backend.writestream = pgit_odb_backend_writestream;
+//  custom_backend->backend.readstream = pgit_odb_backend_readstream;
+    if (PyIter_Check((PyObject *)self))
+        custom_backend->backend.foreach = pgit_odb_backend_foreach;
 
-    if (PyObject_HasAttrString((PyObject *)self, "read_prefix")) {
-        be->read_prefix = PyObject_GetAttrString(
-                (PyObject *)self, "read_prefix");
-        be->backend.read_prefix = pygit2_odb_backend_read_prefix;
-    }
+    // Cross reference (don't incref because it's something internal)
+    custom_backend->py_backend = (PyObject *)self;
+    self->odb_backend = (git_odb_backend *)custom_backend;
 
-    if (PyObject_HasAttrString((PyObject *)self, "read_header")) {
-        be->read_header = PyObject_GetAttrString(
-                (PyObject *)self, "read_header");
-        be->backend.read_header = pygit2_odb_backend_read_header;
-    }
-
-    if (PyObject_HasAttrString((PyObject *)self, "write")) {
-        be->write = PyObject_GetAttrString(
-                (PyObject *)self, "write");
-        be->backend.write = pygit2_odb_backend_write;
-    }
-
-    /* TODO: Stream-based read/write
-    if (PyObject_HasAttrString((PyObject *)self, "writestream")) {
-        be->writestream = PyObject_GetAttrString(
-                (PyObject *)self, "writestream");
-        be->backend.writestream = pygit2_odb_backend_writestream;
-    }
-
-    if (PyObject_HasAttrString((PyObject *)self, "readstream")) {
-        be->readstream = PyObject_GetAttrString(
-                (PyObject *)self, "readstream");
-        be->backend.readstream = pygit2_odb_backend_readstream;
-    }
-    */
-
-    if (PyObject_HasAttrString((PyObject *)self, "exists")) {
-        be->exists = PyObject_GetAttrString(
-                (PyObject *)self, "exists");
-        be->backend.exists = pygit2_odb_backend_exists;
-    }
-
-    if (PyObject_HasAttrString((PyObject *)self, "exists_prefix")) {
-        be->exists_prefix = PyObject_GetAttrString(
-                (PyObject *)self, "exists_prefix");
-        be->backend.exists_prefix = pygit2_odb_backend_exists_prefix;
-    }
-
-    if (PyObject_HasAttrString((PyObject *)self, "refresh")) {
-        be->refresh = PyObject_GetAttrString(
-                (PyObject *)self, "refresh");
-        be->backend.refresh = pygit2_odb_backend_refresh;
-    }
-
-    if (PyIter_Check((PyObject *)self)) {
-        be->backend.foreach = pygit2_odb_backend_foreach;
-    }
-
-    /* TODO:
-    if (PyObject_HasAttrString((PyObject *)self, "writepack")) {
-        be->writepack = PyObject_GetAttrString(
-                (PyObject *)self, "writepack");
-        be->backend.writepack = pygit2_odb_backend_writepack;
-        Py_INCREF(be->writepack);
-    }
-
-    if (PyObject_HasAttrString((PyObject *)self, "freshen")) {
-        be->freshen = PyObject_GetAttrString(
-                (PyObject *)self, "freshen");
-        be->backend.freshen = pygit2_odb_backend_freshen;
-        Py_INCREF(be->freshen);
-    }
-    */
-
-    Py_INCREF((PyObject *)self);
-    be->backend.free = pygit2_odb_backend_free;
-
-    self->odb_backend = (git_odb_backend *)be;
     return 0;
 }
 
 void
 OdbBackend_dealloc(OdbBackend *self)
 {
-    if (self->odb_backend && self->odb_backend->read == pygit2_odb_backend_read) {
-        struct pygit2_odb_backend *be = (struct pygit2_odb_backend *)self->odb_backend;
-        Py_CLEAR(be->read);
-        Py_CLEAR(be->read_prefix);
-        Py_CLEAR(be->read_header);
-        Py_CLEAR(be->write);
-        Py_CLEAR(be->writestream);
-        Py_CLEAR(be->readstream);
-        Py_CLEAR(be->exists);
-        Py_CLEAR(be->exists_prefix);
-        Py_CLEAR(be->refresh);
-        Py_CLEAR(be->writepack);
-        Py_CLEAR(be->freshen);
-        free(be);
+    if (self->odb_backend && self->odb_backend->read == pgit_odb_backend_read) {
+        pgit_odb_backend *custom_backend = (pgit_odb_backend *)self->odb_backend;
+        free(custom_backend);
     }
 
     Py_TYPE(self)->tp_free((PyObject *) self);
@@ -407,6 +295,8 @@ OdbBackend_build_as_iter(const git_oid *oid, void *accum)
         return GIT_EUSER;
 
     err = PyList_Append((PyObject*)accum, py_oid);
+    Py_DECREF(py_oid);
+
     if (err < 0)
         return GIT_EUSER;
 
@@ -416,24 +306,23 @@ OdbBackend_build_as_iter(const git_oid *oid, void *accum)
 PyObject *
 OdbBackend_as_iter(OdbBackend *self)
 {
-    int err;
     PyObject *accum = PyList_New(0);
-    PyObject *ret = NULL;
+    PyObject *iter = NULL;
 
-    err = self->odb_backend->foreach(self->odb_backend,
-            OdbBackend_build_as_iter, (void*)accum);
+    int err = self->odb_backend->foreach(self->odb_backend, OdbBackend_build_as_iter, (void*)accum);
     if (err == GIT_EUSER)
         goto exit;
+
     if (err < 0) {
-        ret = Error_set(err);
+        Error_set(err);
         goto exit;
     }
 
-    ret = PyObject_GetIter(accum);
+    iter = PyObject_GetIter(accum);
 
 exit:
     Py_DECREF(accum);
-    return ret;
+    return iter;
 }
 
 PyDoc_STRVAR(OdbBackend_read__doc__,
@@ -447,26 +336,23 @@ OdbBackend_read(OdbBackend *self, PyObject *py_hex)
     int err;
     git_oid oid;
     git_object_t type;
-    size_t len, sz;
+    size_t len, size;
     void *data;
-    PyObject* tuple;
 
-    if (self->odb_backend->read == NULL) {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
-    }
+    if (self->odb_backend->read == NULL)
+        Py_RETURN_NOTIMPLEMENTED;
 
     len = py_oid_to_git_oid(py_hex, &oid);
     if (len == 0)
         return NULL;
 
-    err = self->odb_backend->read(&data, &sz, &type, self->odb_backend, &oid);
+    err = self->odb_backend->read(&data, &size, &type, self->odb_backend, &oid);
     if (err != 0) {
         Error_set_oid(err, &oid, len);
         return NULL;
     }
 
-    tuple = Py_BuildValue("(ny#)", type, data, sz);
+    PyObject *tuple = Py_BuildValue("(ny#)", type, data, size);
 
     git_odb_backend_data_free(self->odb_backend, data);
 
@@ -484,31 +370,27 @@ OdbBackend_read_prefix(OdbBackend *self, PyObject *py_hex)
     int err;
     git_oid oid, oid_out;
     git_object_t type;
-    size_t len, sz;
+    size_t len, size;
     void *data;
-    PyObject *tuple, *py_oid_out;
 
-    if (self->odb_backend->read_prefix == NULL) {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
-    }
+    if (self->odb_backend->read_prefix == NULL)
+        Py_RETURN_NOTIMPLEMENTED;
 
     len = py_oid_to_git_oid(py_hex, &oid);
     if (len == 0)
         return NULL;
 
-    err = self->odb_backend->read_prefix(&oid_out,
-            &data, &sz, &type, self->odb_backend, &oid, len);
+    err = self->odb_backend->read_prefix(&oid_out, &data, &size, &type, self->odb_backend, &oid, len);
     if (err != 0) {
         Error_set_oid(err, &oid, len);
         return NULL;
     }
 
-    py_oid_out = git_oid_to_python(&oid_out);
-    if (py_oid_out == NULL) {
+    PyObject *py_oid_out = git_oid_to_python(&oid_out);
+    if (py_oid_out == NULL)
         return Error_set_exc(PyExc_MemoryError);
-    }
-    tuple = Py_BuildValue("(ny#O)", type, data, sz, py_oid_out);
+
+    PyObject *tuple = Py_BuildValue("(ny#N)", type, data, size, py_oid_out);
 
     git_odb_backend_data_free(self->odb_backend, data);
 
@@ -528,10 +410,8 @@ OdbBackend_read_header(OdbBackend *self, PyObject *py_hex)
     git_object_t type;
     git_oid oid;
 
-    if (self->odb_backend->read_header == NULL) {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
-    }
+    if (self->odb_backend->read_header == NULL)
+        Py_RETURN_NOTIMPLEMENTED;
 
     len = py_oid_to_git_oid(py_hex, &oid);
     if (len == 0)
@@ -558,10 +438,8 @@ OdbBackend_exists(OdbBackend *self, PyObject *py_hex)
     size_t len;
     git_oid oid;
 
-    if (self->odb_backend->exists == NULL) {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
-    }
+    if (self->odb_backend->exists == NULL)
+        Py_RETURN_NOTIMPLEMENTED;
 
     len = py_oid_to_git_oid(py_hex, &oid);
     if (len == 0)
@@ -589,18 +467,15 @@ OdbBackend_exists_prefix(OdbBackend *self, PyObject *py_hex)
     size_t len;
     git_oid oid;
 
-    if (self->odb_backend->exists_prefix == NULL) {
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
-    }
+    if (self->odb_backend->exists_prefix == NULL)
+        Py_RETURN_NOTIMPLEMENTED;
 
     len = py_oid_to_git_oid(py_hex, &oid);
     if (len == 0)
         return NULL;
 
     git_oid out;
-    result = self->odb_backend->exists_prefix(&out,
-            self->odb_backend, &oid, len);
+    result = self->odb_backend->exists_prefix(&out, self->odb_backend, &oid, len);
 
     if (result < 0)
         return Error_set(result);
@@ -621,11 +496,9 @@ PyDoc_STRVAR(OdbBackend_refresh__doc__,
 PyObject *
 OdbBackend_refresh(OdbBackend *self)
 {
-    if (self->odb_backend->refresh == NULL) {
-        /* XXX: Should we no-op instead? */
-        Py_INCREF(Py_NotImplemented);
-        return Py_NotImplemented;
-    }
+    if (self->odb_backend->refresh == NULL)
+        Py_RETURN_NOTIMPLEMENTED;
+
     self->odb_backend->refresh(self->odb_backend);
     Py_RETURN_NONE;
 }
@@ -692,14 +565,13 @@ PyTypeObject OdbBackendType = {
 };
 
 PyObject *
-wrap_odb_backend(git_odb_backend *c_odb_backend)
+wrap_odb_backend(git_odb_backend *odb_backend)
 {
-    OdbBackend *pygit2_odb_backend = PyObject_New(OdbBackend, &OdbBackendType);
+    OdbBackend *py_backend = PyObject_New(OdbBackend, &OdbBackendType);
+    if (py_backend)
+        py_backend->odb_backend = odb_backend;
 
-    if (pygit2_odb_backend)
-        pygit2_odb_backend->odb_backend = c_odb_backend;
-
-    return (PyObject *)pygit2_odb_backend;
+    return (PyObject *)py_backend;
 }
 
 PyDoc_STRVAR(OdbBackendPack__doc__, "Object database backend for packfiles.");
