@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2020 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -602,6 +602,9 @@ PyInit__pygit2(void)
     ADD_CONSTANT_INT(m, GIT_MERGE_ANALYSIS_UP_TO_DATE)
     ADD_CONSTANT_INT(m, GIT_MERGE_ANALYSIS_FASTFORWARD)
     ADD_CONSTANT_INT(m, GIT_MERGE_ANALYSIS_UNBORN)
+    ADD_CONSTANT_INT(m, GIT_MERGE_PREFERENCE_NONE)
+    ADD_CONSTANT_INT(m, GIT_MERGE_PREFERENCE_NO_FASTFORWARD)
+    ADD_CONSTANT_INT(m, GIT_MERGE_PREFERENCE_FASTFORWARD_ONLY)
 
     /* Describe */
     ADD_CONSTANT_INT(m, GIT_DESCRIBE_DEFAULT);
