@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2020 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -264,6 +264,19 @@ Repository_is_bare__get__(Repository *self)
 }
 
 
+PyDoc_STRVAR(Repository_is_shallow__doc__,
+  "Check if a repository is a shallow repository.");
+
+PyObject *
+Repository_is_shallow__get__(Repository *self)
+{
+    if (git_repository_is_shallow(self->repo) > 0)
+        Py_RETURN_TRUE;
+
+    Py_RETURN_FALSE;
+}
+
+
 PyDoc_STRVAR(Repository_git_object_lookup_prefix__doc__,
   "git_object_lookup_prefix(oid) -> Object\n"
   "\n"
@@ -700,31 +713,33 @@ Repository_cherrypick(Repository *self, PyObject *py_oid)
 }
 
 PyDoc_STRVAR(Repository_walk__doc__,
-  "walk(oid[, sort_mode]) -> iterator\n"
-  "\n"
-  "Generator that traverses the history starting from the given commit.\n"
-  "The following types of sorting could be used to control traversing\n"
-  "direction:\n"
-  "\n"
-  "* GIT_SORT_NONE. This is the default sorting for new walkers.\n"
-  "  Sort the repository contents in no particular ordering\n"
-  "* GIT_SORT_TOPOLOGICAL. Sort the repository contents in topological order\n"
-  "  (parents before children); this sorting mode can be combined with\n"
-  "  time sorting.\n"
-  "* GIT_SORT_TIME. Sort the repository contents by commit time\n"
-  "* GIT_SORT_REVERSE. Iterate through the repository contents in reverse\n"
-  "  order; this sorting mode can be combined with any of the above.\n"
-  "\n"
-  "Example:\n"
-  "\n"
-  "  >>> from pygit2 import Repository\n"
-  "  >>> from pygit2 import GIT_SORT_TOPOLOGICAL, GIT_SORT_REVERSE\n"
-  "  >>> repo = Repository('.git')\n"
-  "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL):\n"
-  "  ...    print(commit.message)\n"
-  "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL | GIT_SORT_REVERSE):\n"
-  "  ...    print(commit.message)\n"
-  "  >>>\n");
+    "walk(oid[, sort_mode]) -> iterator\n"
+    "\n"
+    "Generator that traverses the history starting from the given commit.\n"
+    "The following types of sorting could be used to control traversing\n"
+    "direction:\n"
+    "\n"
+    "* GIT_SORT_NONE. Sort the output with the same default method from\n"
+    "  `git`: reverse chronological order. This is the default sorting for\n"
+    "  new walkers.\n"
+    "* GIT_SORT_TOPOLOGICAL. Sort the repository contents in topological order\n"
+    "  (no parents before all of its children are shown); this sorting mode\n"
+    "  can be combined with time sorting to produce `git`'s `--date-order``.\n"
+    "* GIT_SORT_TIME. Sort the repository contents by commit time; this sorting\n"
+    "  mode can be combined with topological sorting.\n"
+    "* GIT_SORT_REVERSE.  Iterate through the repository contents in reverse\n"
+    "  order; this sorting mode can be combined with any of the above.\n"
+    "\n"
+    "Example:\n"
+    "\n"
+    "  >>> from pygit2 import Repository\n"
+    "  >>> from pygit2 import GIT_SORT_TOPOLOGICAL, GIT_SORT_REVERSE\n"
+    "  >>> repo = Repository('.git')\n"
+    "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL):\n"
+    "  ...    print(commit.message)\n"
+    "  >>> for commit in repo.walk(repo.head.target, GIT_SORT_TOPOLOGICAL | GIT_SORT_REVERSE):\n"
+    "  ...    print(commit.message)\n"
+    "  >>>\n");
 
 PyObject *
 Repository_walk(Repository *self, PyObject *args)
@@ -1299,25 +1314,24 @@ static int foreach_path_cb(git_submodule *submodule, const char *name, void *pay
     PyObject *list = (PyObject *)payload;
     PyObject *path = to_unicode(git_submodule_path(submodule), NULL, NULL);
 
-    return PyList_Append(list, path);
+    int err = PyList_Append(list, path);
+    Py_DECREF(path);
+    return err;
 }
 
 PyObject *
 Repository_listall_submodules(Repository *self, PyObject *args)
 {
-    int err;
-    PyObject *list;
-
-    list = PyList_New(0);
+    PyObject *list = PyList_New(0);
     if (list == NULL)
         return NULL;
 
-    err = git_submodule_foreach(self->repo, foreach_path_cb, list);
+    int err = git_submodule_foreach(self->repo, foreach_path_cb, list);
     if (err) {
         Py_DECREF(list);
-        if (PyErr_Occurred()) {
-          return NULL;
-        }
+        if (PyErr_Occurred())
+            return NULL;
+
         return Error_set(err);
     }
 
@@ -1344,7 +1358,6 @@ Repository_init_submodules(Repository* self, PyObject *args, PyObject *kwds)
     PyObject *oflag = Py_False;
     char *kwlist[] = {"submodules", "overwrite", NULL};
     int err;
-    PyObject *iter, *next;
     const char *c_subpath;
     git_submodule *submodule;
 
@@ -1363,32 +1376,42 @@ Repository_init_submodules(Repository* self, PyObject *args, PyObject *kwds)
         Py_RETURN_NONE;
     }
 
-    iter = PyObject_GetIter(list);
+    PyObject *iter = PyObject_GetIter(list);
     if (!iter)
         return NULL;
 
+    PyObject *next = NULL;
     while (1) {
+        Py_XDECREF(next); // Decref from the previous iteration
         next = PyIter_Next(iter);
-        if (!next)
+        if (next == NULL) // List exhausted
             break;
 
         c_subpath = pgit_borrow(next);
         if (c_subpath == NULL)
-            return NULL;
+            goto error;
 
         err = git_submodule_lookup(&submodule, self->repo, c_subpath);
         if (!submodule) {
             PyErr_SetString(PyExc_KeyError, "Submodule does not exist");
-            return NULL;
+            goto error;
         }
 
         err = git_submodule_init(submodule, fflag);
         if (err) {
-            return Error_set(err);
+            Error_set(err);
+            goto error;
         }
     }
 
+    // Success
+    Py_DECREF(iter);
     Py_RETURN_NONE;
+
+error:
+    Py_DECREF(iter);
+    Py_XDECREF(next);
+    return NULL;
 }
 
 PyDoc_STRVAR(Repository_lookup_reference__doc__,
@@ -1446,7 +1469,7 @@ Repository_lookup_reference_dwim(Repository *self, PyObject *py_name)
 }
 
 PyDoc_STRVAR(Repository_create_reference_direct__doc__,
-  "create_reference_direct(name, target, force)\n"
+  "create_reference_direct(name, target, force, message=None)\n"
   "\n"
   "Create a new reference \"name\" which points to an object.\n"
   "\n"
@@ -1458,6 +1481,9 @@ PyDoc_STRVAR(Repository_create_reference_direct__doc__,
   "    If True references will be overridden, otherwise (the default) an\n"
   "    exception is raised.\n"
   "\n"
+  "message\n"
+  "    Optional message to use for the reflog.\n"
+  "\n"
   "Examples::\n"
   "\n"
   "    repo.create_reference_direct('refs/heads/foo', repo.head.target, False)");
@@ -1471,15 +1497,18 @@ Repository_create_reference_direct(Repository *self,  PyObject *args,
     char *c_name;
     git_oid oid;
     int err, force;
+    const char *message = NULL;
+    char *keywords[] = {"name", "target", "force", "message", NULL};
 
-    if (!PyArg_ParseTuple(args, "sOi", &c_name, &py_obj, &force))
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "sOi|z", keywords,
+                                     &c_name, &py_obj, &force, &message))
         return NULL;
 
     err = py_oid_to_git_oid_expand(self->repo, py_obj, &oid);
     if (err < 0)
         return NULL;
 
-    err = git_reference_create(&c_reference, self->repo, c_name, &oid, force, NULL);
+    err = git_reference_create(&c_reference, self->repo, c_name, &oid, force, message);
     if (err < 0)
         return Error_set(err);
 
@@ -1487,7 +1516,7 @@ Repository_create_reference_direct(Repository *self,  PyObject *args,
 }
 
 PyDoc_STRVAR(Repository_create_reference_symbolic__doc__,
-  "create_reference_symbolic(name, source, force)\n"
+  "create_reference_symbolic(name, source, force, message=None)\n"
   "\n"
   "Create a new reference \"name\" which points to another reference.\n"
   "\n"
@@ -1499,6 +1528,9 @@ PyDoc_STRVAR(Repository_create_reference_symbolic__doc__,
   "    If True references will be overridden, otherwise (the default) an\n"
   "    exception is raised.\n"
   "\n"
+  "message\n"
+  "    Optional message to use for the reflog.\n"
+  "\n"
   "Examples::\n"
   "\n"
   "    repo.create_reference_symbolic('refs/tags/foo', 'refs/heads/master', False)");
@@ -1510,12 +1542,15 @@ Repository_create_reference_symbolic(Repository *self,  PyObject *args,
     git_reference *c_reference;
     char *c_name, *c_target;
     int err, force;
+    const char *message = NULL;
+    char *keywords[] = {"name", "target", "force", "message", NULL};
 
-    if (!PyArg_ParseTuple(args, "ssi", &c_name, &c_target, &force))
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "ssi|z", keywords,
+                                     &c_name, &c_target, &force, &message))
         return NULL;
 
     err = git_reference_symbolic_create(&c_reference, self->repo, c_name,
-                                        c_target, force, NULL);
+                                        c_target, force, message);
     if (err < 0)
         return Error_set(err);
 
@@ -1743,26 +1778,26 @@ PyDoc_STRVAR(Repository_notes__doc__, "");
 PyObject *
 Repository_notes(Repository *self, PyObject *args)
 {
-    NoteIter *iter = NULL;
     char *ref = "refs/notes/commits";
-    int err = GIT_ERROR;
-
     if (!PyArg_ParseTuple(args, "|s", &ref))
         return NULL;
 
-    iter = PyObject_New(NoteIter, &NoteIterType);
-    if (iter != NULL) {
-        iter->repo = self;
-        iter->ref = ref;
+    NoteIter *iter = PyObject_New(NoteIter, &NoteIterType);
+    if (iter == NULL)
+        return NULL;
 
-        err = git_note_iterator_new(&iter->iter, self->repo, iter->ref);
-        if (err == GIT_OK) {
-            Py_INCREF(self);
-            return (PyObject*)iter;
-        }
+    Py_INCREF(self);
+    iter->repo = self;
+    iter->ref = ref;
+    iter->iter = NULL;
+
+    int err = git_note_iterator_new(&iter->iter, self->repo, iter->ref);
+    if (err != GIT_OK) {
+        Py_DECREF(iter);
+        return Error_set(err);
     }
 
-    return Error_set(err);
+    return (PyObject*)iter;
 }
 
 
@@ -2086,8 +2121,8 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, cherrypick, METH_O),
     METHOD(Repository, apply, METH_VARARGS),
     METHOD(Repository, applies, METH_O),
-    METHOD(Repository, create_reference_direct, METH_VARARGS),
-    METHOD(Repository, create_reference_symbolic, METH_VARARGS),
+    METHOD(Repository, create_reference_direct, METH_VARARGS | METH_KEYWORDS),
+    METHOD(Repository, create_reference_symbolic, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, compress_references, METH_NOARGS),
     METHOD(Repository, listall_references, METH_NOARGS),
     METHOD(Repository, raw_listall_references, METH_NOARGS),
@@ -2130,6 +2165,7 @@ PyGetSetDef Repository_getseters[] = {
     GETTER(Repository, head_is_unborn),
     GETTER(Repository, is_empty),
     GETTER(Repository, is_bare),
+    GETTER(Repository, is_shallow),
     GETSET(Repository, workdir),
     GETTER(Repository, default_signature),
     GETTER(Repository, odb),
