@@ -1,4 +1,4 @@
-# Copyright 2010-2020 The pygit2 contributors
+# Copyright 2010-2021 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -633,3 +633,12 @@ def test_open_extended(tmp_path):
         assert repo.is_bare
         assert repo.path == orig_repo.path
         assert not repo.workdir
+
+def test_is_shallow(testrepo):
+    assert not testrepo.is_shallow
+
+    # create a dummy shallow file
+    with open(os.path.join(testrepo.path, 'shallow'), 'wt') as f:
+        f.write('abcdef0123456789abcdef0123456789abcdef00\n')
+
+    assert testrepo.is_shallow
